%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% IritPlotSolution.m - MATLAB interface for plotting solutions returned by    %
% IritPolynomialSolver.	                                                      %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% (C) Gershon Elber, Technion, Israel Institute of Technology                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Written by Ron Zisman, Dec. 13.                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% DESCRIPTION:
%   Plots a figure for a given solution based on its type.
% PARAMETERS:
%   Solution:  A solution struct containing points, polylines or triangles. 
%	       WARNING: IritPlotSolution assumes a valid solution returned by IritPolymonialSolver!
%   Dimensions:  Chosen dimensions to be plotted if solution exceeds 3 dimensions.
% RETURN VALUE:
%   void
% SEE ALSO:
%   IritPointPlot, IritPolyPlot, IritTrianglePlot
% KEYWORDS:
%   IritPlotSolution

function IritPlotSolution(Solution, varargin)
	
    P = inputParser;
    P.FunctionName = 'IritPlotSolution';
    DefaultDimensions = [1 2 3];
    ValidateDimensions = @(x) validateattributes(x, {'numeric'}, {'row', 'integer', '>=', 1, '<=', Solution.Dimension, 'numel', 3});
    addRequired(P, 'Solution');
    addOptional(P, 'Dimensions', DefaultDimensions, ValidateDimensions);
    parse(P, Solution, varargin{:});
    Properties = P.Results;
    
    if strcmp(Solution.Type, 'POINT')
	IritPointPlot(Solution, Properties.Dimensions);
    elseif strcmp(Solution.Type, 'POLY')
	IritPolyPlot(Solution, Properties.Dimensions);		
    elseif strcmp(Solution.Type, 'TRIANGLE')
	IritTrianglePlot(Solution, Properties.Dimensions);
    end
end


%%%%%%%%%%%%%%%%%%%%%%%
%%% Local functions %%%
%%%%%%%%%%%%%%%%%%%%%%%

function IritPointPlot(Solution, Dims)
% DESCRIPTION:
%   Plots point solution.
% PARAMETERS:
%   Solution:  A solution struct containing points.
%   Dims:  Chosen dimensions to be plotted if solution exceeds 3 dimensions.
% RETURN VALUE:
%   void

    if Solution.Dimension == 1
	return;
    end
    if Solution.Dimension == 2	
	% 2D plot %
	X = Solution.Data(:, 1);
	Y = Solution.Data(:, 2);
	plot(X, Y, 'r*');
% 	axis([Solution.LowerBounds(1) Solution.UpperBounds(1) Solution.LowerBounds(2) Solution.UpperBounds(2)]);
	title('Solution Points 2D', 'fontsize', 10, 'FontWeight', 'bold');
	xlabel('x1', 'FontWeight', 'bold');
	ylabel('x2', 'FontWeight', 'bold');
	grid on;
    else
	% 3D plot %
	X = Solution.Data(:, Dims(1));
	Y = Solution.Data(:, Dims(2));
	Z = Solution.Data(:, Dims(3));
	plot3(X, Y, Z, 'r*');
% 	axis([Solution.LowerBounds(1) Solution.UpperBounds(1) Solution.LowerBounds(2) Solution.UpperBounds(2) Solution.LowerBounds(3) Solution.UpperBounds(3)]);
	title('Solution Points 3D', 'fontsize', 10, 'FontWeight', 'bold');
	xlabel(sprintf('x%d', Dims(1)), 'FontWeight', 'bold');
	ylabel(sprintf('x%d', Dims(2)), 'FontWeight', 'bold');
	zlabel(sprintf('x%d', Dims(3)), 'FontWeight', 'bold');
	grid on;
    end
end

function IritPolyPlot(Solution, Dims)
% DESCRIPTION:
%   Plots polyline solution.
% PARAMETERS:
%   Solution:  A solution struct containing polylines.
%   Dims:  Chosen dimensions to be plotted if solution exceeds 3 dimensions.
% RETURN VALUE:
%   void

    NumPolys = size(Solution.Data, 1);
    %figure;
    %colormap(winter);
    if Solution.Dimension == 2
	% 2D plot %
	for i = 1:NumPolys	
	    X = Solution.Data{i}(:, 1);
	    Y = Solution.Data{i}(:, 2);
	    plot(X, Y, 'r', 'LineWidth', 2);
	    hold on;
    end
	title('Solution Polylines 2D', 'fontsize', 10, 'FontWeight', 'bold');
	xlabel('x1', 'FontWeight', 'bold');
	ylabel('x2', 'FontWeight', 'bold');
	grid on;
    else	
	% 3D plot %
    for i = 1:NumPolys
	    X = Solution.Data{i}(:, Dims(1));
	    Y = Solution.Data{i}(:, Dims(2));
	    Z = Solution.Data{i}(:, Dims(3));
	    plot3(X, Y, Z, 'r', 'LineWidth', 2);
	    hold on;
    end
	title('Solution Polylines 3D', 'fontsize', 10, 'FontWeight', 'bold');
	xlabel(sprintf('x%d', Dims(1)), 'FontWeight', 'bold');
	ylabel(sprintf('x%d', Dims(2)), 'FontWeight', 'bold');
	zlabel(sprintf('x%d', Dims(3)), 'FontWeight', 'bold');
	grid on;
    end
end

function IritTrianglePlot(Solution, Dims)
% DESCRIPTION:
%   Plots triangle solution.
% PARAMETERS:
%   Solution:  A solution struct containing triangles.
%   Dims:  Chosen dimensions to be plotted if solution exceeds 3 dimensions.
% RETURN VALUE:
%   void

    NumTriangles = size(Solution.Data, 1);
    % 3D plot %
    figure;
    colormap(summer);
    X = [];
    Y = [];
    Z = [];
    Tri = [];
    for i = 1:NumTriangles
	X = [X; Solution.Data{i}(:, Dims(1))];
	Y = [Y; Solution.Data{i}(:, Dims(2))];
	Z = [Z; Solution.Data{i}(:, Dims(3))];
	Tri = [Tri; [3 * i - 2, 3 * i - 1, 3 * i]];
    end
    trisurf(Tri, X, Y, Z);
%     axis([Solution.LowerBounds(1) Solution.UpperBounds(1) Solution.LowerBounds(2) Solution.UpperBounds(2) Solution.LowerBounds(3) Solution.UpperBounds(3)]);
    title('Solution Triangles 3D', 'fontsize', 10, 'FontWeight', 'bold');
    xlabel(sprintf('x%d', Dims(1)), 'FontWeight', 'bold');
    ylabel(sprintf('x%d', Dims(2)), 'FontWeight', 'bold');
    zlabel(sprintf('x%d', Dims(3)), 'FontWeight', 'bold');
    grid on;
end
