%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% IritPolynomialSolver.m - MATLAB interface for solving polynomial            %
% equations.                                                                  %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% (C) Gershon Elber, Technion, Israel Institute of Technology                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Written by Ron Zisman, Dec. 13.                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% DESCRIPTION:
%   Solves a polynomial equation and plots the solution.
%   Note: If solution dimension exceeds 3, plots according to the first 3 dimensions.
% PARAMETERS:
%   VarTotal:  Total number of variables used in the equations.
%   Equations:  A cell array of strings, where each string is an equation defined as a symbolic expression.
%		Note: The solver assumes only left side equations - the right side is always 0.
%   LowerBounds:  A vector of doubles to specify the lower bound of each variable from x1 to the last, respectively.
%		  The default lower bound for all variables is 0. (Optional)
%   UpperBounds:  A vector of doubles to specify the upper bound of each variable from x1 to the last, respectively.
%		  The default upper bound for all variables is 1. (Optional)
%   Constraints:  A vector of values -1 (negative), 0 (zero), or 1 (positive) to specify the sign of the equation.
%		  The default vector is all zeros. (Optional)
%   NumericTol:  The numeric tolerance for the solution. 
%		 The default value is 1e-10. (Optional)
%   SubdivTol:  The subdivision tolerance for the solution.
%		The default value is 1e-3. (Optional)
%   Step: The step used in curve tracing, when the solution is a
%       (univariate) polyline. The default value is 0.1 * Subdivision tolerance (Optional).
%       Note: If the plots are slow, use a bigger step value.
% RETURN VALUE:
%   OutSolution:  A solution struct of points, polylines, triangles, or null. 
%                 Type - 'NULL', 'POINT', 'POLY', or 'TRIANGLE'.
%                 Data - Each solution data is represented differently:
%                        Points: A double type matrix of points as rows and dimensions as columns.
%                        Polylines: A cell array of Points. Each cell is made of a finite number of points.
%                        Triangles: A cell matrix of triangles as rows, vertices as the first column and normals as the second column.
%			            Each cell is a Points matrix of 3 points.
%			            Note: Normals might not be available.
%                 Dimension - The dimnesion of the solution. 
%                 LowerBounds - See LowerBounds in PARAMETERS.
%                 UpperBounds - See UpperBounds in PARAMETERS.
% SEE ALSO:
%   IritPolynomialSolverMex, IritPlotSolution, IritGetInput, IritEquationParser
% KEYWORDS:
%   IritPolynomialSolver
function OutSolution = IritPolynomialSolver(VarTotal, Equations, varargin)	
    tic;
    fprintf('\n\nThis Irit polynomial solver is (C) Gershon Elber, Technion, Israel Institute of Technology.\n<a href="http://www.cs.technion.ac.il/~irit">www.cs.technion.ac.il/~irit</a>\n\n');

    % Define solution type. %
    SolType = struct('NULL', 0, 'POINT', 1, 'POLY', 2, 'TRIANGLE', 3);
	
    % Set user properties. %
    Properties = IritGetInput(VarTotal, Equations, varargin{:});
	
    % Parse equations. %
    [PowersMat, CoeffsArray, MaxPowersArray] = IritEquationParser(VarTotal, Equations);
    t1 = toc;
    fprintf('Input processing and parsing time: %.4f\n', t1);
    % Call mex function to receive solution. %
    tic;
    [Sol, T] = IritPolynomialSolverMex(VarTotal, PowersMat, CoeffsArray, MaxPowersArray, Properties.LowerBounds, Properties.UpperBounds, Properties.Constraints, Properties.NumericTol, Properties.SubdivTol, Properties.Step);
    t = toc;
    fprintf('Equation solving time: %.4f\n', t);
    % Handle solution. %
    switch (T)
	case SolType.POINT
	    OutSolution.Type = 'POINT';
	    NumPoints = size(Sol, 1);
	    NumDims = size(Sol, 2);
	    OutSolution.Dimension = NumDims;
	    fprintf('Found %d solution points:\n', NumPoints);
	    % Print solution points to screen. %
	    for i = 1:NumPoints
		fprintf('%d. (', i);
		for j = 1:(NumDims - 1)
                    fprintf('%f, ', Sol(i, j));
		end
		fprintf('%f)\n', Sol(i, NumDims));
	    end
	case SolType.POLY
	    OutSolution.Type = 'POLY';
	    OutSolution.Dimension = size(Sol{1}, 2);
	    NumPolys = size(Sol, 1);
	    fprintf('Found %d connected components.\n', NumPolys);
	case SolType.TRIANGLE
	    OutSolution.Type = 'TRIANGLE';
	    OutSolution.Dimension = size(Sol{1}, 2);
	    NumTriangles = size(Sol, 1);
	    fprintf('Found %d triangles.\n', NumTriangles);
	case SolType.NULL
	    OutSolution.Type = 'NULL';
	    fprintf('No solution found.\n');
    end
	
    OutSolution.Data = Sol;
    OutSolution.LowerBounds = Properties.LowerBounds;
    OutSolution.UpperBounds = Properties.UpperBounds;
    IritPlotSolution(OutSolution);
end


%%%%%%%%%%%%%%%%%%%%%%%
%%% Local functions %%%
%%%%%%%%%%%%%%%%%%%%%%%

function Properties = IritGetInput(VarTotal, Equations, varargin)
    P = inputParser;
    P.FunctionName = 'IritGetInput';
	
    % Initialize default values. %
    DefaultLowerBounds = zeros(1, VarTotal);
    DefaultUpperBounds = zeros(1, VarTotal) + 1;
    DefaultConstraints = zeros(1, numel(Equations));
    DefaultNumericTol = 1e-10;
    DefaultSubdivTol = 1e-3;
    DefaultStep = 0.1 * DefaultSubdivTol;
	
    % Set validations. %
    ValidateVarTotal = @(x) validateattributes(x, {'numeric'}, {'nonempty', 'integer', 'positive'});
    ValidateEquations = @(x) validateattributes(x, {'cell'}, {'vector'});
    ValidateBounds = @(x) validateattributes(x, {'double'}, {'vector', 'numel', VarTotal});
    ValidateConstraints = @(x) validateattributes(x, {'numeric'}, {'vector', 'integer', '>=', -1, '<=', 1, 'numel', VarTotal});
    ValidateTolerance = @(x) validateattributes(x, {'double'}, {'scalar'});
	
    % Add parameters. %
    addRequired(P, 'VarTotal', ValidateVarTotal);
    addRequired(P, 'Equations', ValidateEquations);
    addParamValue(P, 'LowerBounds', DefaultLowerBounds, ValidateBounds);
    addParamValue(P, 'UpperBounds', DefaultUpperBounds, ValidateBounds);
    addParamValue(P, 'Constraints', DefaultConstraints, ValidateConstraints);
    addParamValue(P, 'NumericTol', DefaultNumericTol, ValidateTolerance);
    addParamValue(P, 'SubdivTol', DefaultSubdivTol, ValidateTolerance);
    addParamValue(P, 'Step', DefaultStep, ValidateTolerance);
	
    parse(P, VarTotal, Equations, varargin{:});
    Properties = P.Results;
end	

function [OutPowersMat, OutCoeffsArray, OutMaxPowersArray] = IritEquationParser(VarTotal, Equations)
    for i = 1:numel(Equations)
	
	% Expand equation to monomial form. %
	TempEqn = vpa(expand(sym(Equations{i})));
	if isempty(TempEqn)
	    error('Equation %d is empty.', i);
	end
    
	% Convert equation to string and separate to monomials. %
	StrEqn = char(TempEqn);
	StrEqn = regexprep(StrEqn, '\+ ', '\+');
	StrEqn = regexprep(StrEqn, '\- ', '\-');
	Monoms = regexp(StrEqn, ' ', 'split');
		
	% Initialize the powers matrix and the max powers array. %
	PowersMat{i} = uint32(zeros(VarTotal, numel(Monoms)));
	MaxPowersArray{i} = uint32(zeros(VarTotal, 1));
    
	for j = 1:numel(Monoms)
	
	    % Separate monomial by '*'. % 
	    MonomParts = regexp(Monoms{j}, '\*', 'split');
	    CoeffsArray{i}(j) = 1;
	    for k = 1:numel(MonomParts)
	
		% Get the monomial coefficient. %
		if MonomParts{k}(1) == '+'
		    MonomParts{k} = MonomParts{k}(2:end);
		end
		if MonomParts{k}(1) == '-'
		    MonomParts{k} = MonomParts{k}(2:end);
		    CoeffsArray{i}(j) = -1;
		end
		if isstrprop(MonomParts{k}(1), 'digit') == 1
		    CoeffsArray{i}(j) = CoeffsArray{i}(j) * str2num(MonomParts{k});
		    continue
		end
				
		% Get variable's ID and its power. %
		if isletter(MonomParts{k}(1)) && MonomParts{k}(1) ~= 'x'
		    error('Invalid variable in equation %d. Variables must be of Xi form.', i);
		end
		if MonomParts{k}(1) == 'x'
		    if isstrprop(MonomParts{k}(2), 'digit') == 0
		        error('Invalid variable in equation %d. Variables must be of Xi form.', i);
		    end
		    PowerStr = regexp(MonomParts{k}(2:end), '\^', 'split');
		    if numel(PowerStr) == 1
		        PowerValue = uint32(1);
		    else
			TempPowerValue = str2num(PowerStr{2});
			if rem(TempPowerValue, 1) ~= 0 || TempPowerValue < 1
			    error('Power value of x%d in equation %d must be a natural number.', str2num(PowerStr{1}), i);
			end
			PowerValue = uint32(TempPowerValue);
		    end
		    if str2num(PowerStr{1}) > VarTotal
		        error('Equation %d contains an out of bound variable.', i);
		    end
		    PowersMat{i}(str2num(PowerStr{1}), j) = PowerValue;
					
		    % Update the max power of the monomial accordingly. %
		    if PowerValue > MaxPowersArray{i}(str2num(PowerStr{1}))
			MaxPowersArray{i}(str2num(PowerStr{1})) = PowerValue;
		    end
		end
	    end
	end
    end
	
    OutPowersMat = PowersMat;
    OutCoeffsArray = CoeffsArray;
    OutMaxPowersArray = MaxPowersArray;
end
