%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% IritPolynomialSolverTest.m - MATLAB tester for the IritPolynomialSolver.    %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% (C) Gershon Elber, Technion, Israel Institute of Technology                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Written by Ron Zisman, Dec. 13.                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Presents 5 examples to showcase the IritPolynomialSolver capabilities.
% The plots in examples 1-4 display the input as well as the solution points/curves.
% The plots in examples 5 display two possible projections of the sollution curves.


function IritPolynomialSolverTest()
fprintf('Starting tester.\n');
Handler();
end

function Example1()
x_min = 0; x_max = 1; y_min = 0; y_max = 1; z_min = 0; z_max = 1;
Eq = {'2 * (x1 - 0.5)^2 + (x2 - 0.5)^2 -x3', '0.2 * x1 + 0.4 * x2 - x3 + 0.2'};
fprintf('\nParaboloid equation: %s', Eq{1});
fprintf('\nPlane equation: %s\n\n', Eq{2});
SurfacePlots(1, 2, x_min, x_max, y_min, y_max, z_min, z_max);
Solution = IritPolynomialSolver(3, Eq, 'Step', 1e-3,...
    'LowerBounds', [x_min, y_min, z_min], 'UpperBounds', [x_max, y_max, z_max]);
Solution
Handler();
end

function Example2()
x_min = -0.5; x_max = 1.5; y_min = 0; y_max = 1; z_min = -1.5; z_max = 1;
Eq = {'2 * (x1 - 0.5)^3 + 5 * (x2 - 0.5)^4 - x3 + 0.35', '-(2 * (x1 - 0.5)^2 + (x2 - 0.5)^2 - 0.8) - x3'};
fprintf('\n3x4 degree surface equation: %s', Eq{1});
fprintf('\nParaboloid equation: %s', Eq{2});
SurfacePlots(3, 4, x_min, x_max, y_min, y_max, z_min, z_max);
Solution = IritPolynomialSolver(3, Eq, 'LowerBounds', [x_min, y_min, z_min], 'UpperBounds', [x_max, y_max, z_max]);
Solution
Handler();
end

function Example3()
x_min = 0; x_max = 1; y_min = 0; y_max = 1; 
Eq = {'3 * (x1 - 0.2) * (x1 - 0.4) * (x1 - 0.6) + 0.3 - x2', '5 * (x1 - 0.3) * (x1 - 0.5) * (x1 - 0.9) + 0.4 - x2'};
fprintf('\nFirst cubic curve equation: %s', Eq{1});
fprintf('\nSecond cubic curve equation: %s', Eq{2});
t = x_min : 1e-4 : x_max;
n = length(t);
c1 = zeros(size(t));
c2 = c1;
for i = 1 : n
    c1(i) = 3 * (t(i) - 0.2) * (t(i) - 0.4) * (t(i) - 0.6) + 0.3;
    c2(i) = 5 * (t(i) - 0.3) * (t(i) - 0.5) * (t(i) - 0.9) + 0.4;
end
figure;
axis([x_min, x_max, y_min, y_max]);
axis equal;
hold on;

plot(t, c1, 'b', 'linewidth', 2);
plot(t, c2, 'k', 'linewidth', 2);
Solution = IritPolynomialSolver(2, Eq, 'LowerBounds', [x_min, y_min], 'UpperBounds', [x_max, y_max]);
Solution
Handler();
end

function Example4()
x_min = 0; x_max = 1; y_min = 0; y_max = 1; z_min = 0; z_max = 1;
Eq = {'((x1-0.5)^2) / 0.3^2 + ((x2-0.5)^2) / 0.3^2 + ((x3-0.5)^2) / 0.6^2 - 1', '((x1-0.5)^2) / 0.7^2 + ((x2-0.5)^2) / 0.2^2 + ((x3-0.5)^2) / 0.3^2 - 1', '((x1-0.5)^2) / 0.5^2 + ((x2-0.5)^2) / 0.5^2 + ((x3-0.5)^2) / 0.2^2 - 1'};
fprintf('\nFirst ellipsoid equation: %s', Eq{1});
fprintf('\nSecond ellipsoid equation: %s', Eq{2});
fprintf('\nThird ellipsoid equation: %s\n\n', Eq{3});
figure;
axis([0 1 0 1 0 1]);
axis equal;
hold on;
[x y z] = sphere;
x1 = 0.3 * x + 0.5;
y1 = 0.3 * y + 0.5;
z1 = 0.6 * z + 0.5;
mesh(x1, y1, z1);
x2 = 0.7 * x + 0.5;
y2 = 0.2 * y + 0.5;
z2 = 0.3 * z + 0.5;
mesh(x2, y2, z2);
x3 = 0.5 * x + 0.5;
y3 = 0.5 * y + 0.5;
z3 = 0.2 * z + 0.5;
mesh(x3, y3, z3);
Solution = IritPolynomialSolver(3, Eq, 'LowerBounds', [x_min, y_min, z_min], 'UpperBounds', [x_max, y_max, z_max]);
Solution
Handler();
end

function Example5()
x_min = 0; x_max = 1; y_min = 0; y_max = 1; z_min = 0; z_max = 1; w_min = 0; w_max = 1;
Eq = {'(x1-0.5)^2+(x2-0.5)^2+(x3-0.5)^2-(1-x4+0.5)*(x4-0.5)^2', '(x2-0.5)^2+(x3-0.4)^2 - 1/16', '(x1-0.4)^2+(x4-0.6)^2 - 1/9'};
fprintf('\nDing-Dong surface equation: %s', Eq{1});
fprintf('\nFirst quadratic hyper-surface equation: %s', Eq{2});
fprintf('\nSecond quadratic hyper-surface equation: %s\n\n', Eq{3});
fprintf('\nFigure 1 features dimensions 1, 2 and 3.\nFigure 2 features dimensions 2, 3 and 4.\n\n');
figure;
axis([0 1 0 1 0 1]);
axis equal;
Solution = IritPolynomialSolver(4, Eq, 'Step', 1e-3, 'LowerBounds', [x_min, y_min, z_min, w_min], 'UpperBounds', [x_max, y_max, z_max, w_max]);
figure;
axis([0 1 0 1 0 1]);
axis equal;
hold on; % Draw another figure for other dimensions.
IritPlotSolution(Solution, [2 3 4]);
Solution
Handler();
end

function Handler()
Result = input('Choose [1-6]:\n1 - 3D intersection of a paraboloid and a plane\n2 - 3D intersection a paraboloid and a 3 by 4 degree surface\n3 - 2D intersection of two cubic curves\n4 - 3D intersection of three ellipsoids\n5 - 4D intersection of a ding-dong surface and two quadratic hyper-surfaces\n6 - Exit\n');
switch (Result)
    case 1
        Example1();
    case 2
        Example2();
    case 3
        Example3();
    case 4
        Example4();
    case 5
        Example5();
    case 6
        fprintf('Exiting tester.\n');
    otherwise
        fprintf('Invalid selection.\n');
        Handler();
end
end

function SurfacePlots(SurfNum1, SurfNum2, x_min, x_max, y_min, y_max, z_min, z_max)
x_step = (x_max - x_min) / 30;
y_step = (y_max - y_min) / 30;

x = x_min : x_step : x_max;
y = y_min : y_step : y_max;
nx = length(x);
ny = length(y);
z1 = zeros(nx, ny);
z2 = z1;

for i = 1 : nx
    for j = 1 : ny
        z1(i, j) = EvalZ(x(i), y(j), SurfNum1);
        z2(i, j) = EvalZ(x(i), y(j), SurfNum2);
    end
end
figure;
axis([x_min, x_max, y_min, y_max, z_min, z_max]);
axis equal;
hold on;
surf(x, y, z1');
surf(x, y, z2')
end

function z = EvalZ(x, y, ExNum)

switch ExNum
    case 1
        z = 2 * (x - 0.5)^2 + (y - 0.5)^2;
    case 2
        z = 0.2 * x + 0.4 * y + 0.2;
    case 3
        z = 2 * (x - 0.5)^3 + 5 * (y - 0.5)^4 + 0.35;
    case 4
        z = -(2 * (x - 0.5)^2 + (y - 0.5)^2 - 0.8);
    case 5
        z = 2 * (x - 0.5)^3 + 3 * y^4;
    case 6
        z = -2 * (y - 0.5)^3 - 3 * x^4 + 0.4;
end
end





